Package: spatstat.univar
Version: 3.1-7
Date: 2026-02-18
Title: One-Dimensional Probability Distribution Support for the
        'spatstat' Family
Authors@R: c(person("Adrian", "Baddeley", 
                    role = c("aut", "cre", "cph"),
       	            email = "Adrian.Baddeley@curtin.edu.au",
		    comment = c(ORCID="0000-0001-9499-8382")),
	     person(c("Tilman", "M."), "Davies",
	             role = c("aut", "ctb", "cph"),
		     email = "Tilman.Davies@otago.ac.nz",
		     comment = c(ORCID="0000-0003-0565-1825")),
	     person(c("Martin", "L."), "Hazelton",
	     	    role = c("aut", "ctb", "cph"),
		    email = "Martin.Hazelton@otago.ac.nz",
		    comment = c(ORCID="0000-0001-7831-725X")),
	     person("Ege",   "Rubak", 
                    role = c("aut", "cph"),
		    email = "rubak@math.aau.dk",
		    comment=c(ORCID="0000-0002-6675-533X")),
	     person("Rolf", "Turner", 
                    role = c("aut", "cph"),
 	            email="rolfturner@posteo.net",
		    comment=c(ORCID="0000-0001-5521-5218")),
             person("Greg", "McSwiggan",
	            role = c("ctb", "cph")))
Maintainer: Adrian Baddeley <Adrian.Baddeley@curtin.edu.au>
Depends: R (>= 3.5.0), stats
Imports: spatstat.utils (>= 3.2-2), graphics
Description: Estimation of one-dimensional probability distributions
	     including kernel density estimation, weighted empirical cumulative
	     distribution functions, Kaplan-Meier and reduced-sample estimators
	     for right-censored data, heat kernels, kernel properties,
	     quantiles and integration.
License: GPL (>= 2)
URL: http://spatstat.org/
NeedsCompilation: yes
ByteCompile: true
BugReports: https://github.com/spatstat/spatstat.univar/issues
Packaged: 2026-03-18 07:58:05 UTC; adrian
Author: Adrian Baddeley [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-9499-8382>),
  Tilman M. Davies [aut, ctb, cph] (ORCID:
    <https://orcid.org/0000-0003-0565-1825>),
  Martin L. Hazelton [aut, ctb, cph] (ORCID:
    <https://orcid.org/0000-0001-7831-725X>),
  Ege Rubak [aut, cph] (ORCID: <https://orcid.org/0000-0002-6675-533X>),
  Rolf Turner [aut, cph] (ORCID: <https://orcid.org/0000-0001-5521-5218>),
  Greg McSwiggan [ctb, cph]
Repository: CRAN
Date/Publication: 2026-03-18 09:10:02 UTC
