%% Extra macro definitions for this package
\newcommand{\ournewpaper}{Baddeley, A., Davies, T.M. and Hazelton, M.L. (2025) An improved estimator of the pair correlation function of a spatial point process. \emph{Biometrika} \bold{111}, 2, article asaf021.}
\newcommand{\ournewpapercite}{Baddeley, Davies and Hazelton (2025)}
