%% Extra macro definitions for this package
\newcommand{\ournewpaper}{Baddeley, A., Davies, T.M. and Hazelton, M. (2025) Submitted for publication.}
\newcommand{\ournewpapercite}{Baddeley, Davies and Hazelton (2025)}
